<?php

// sql search
//
// Class for searching sql databases
//
// Copywright (C) 2010 Luke Mounsey
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License included in the file 
// "license.txt" for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

class sql_search{

	// imported properties
	
	public $results=null;
	public $results_display=null;
	public $sql_query=null;
	public $search_query=null;
	public $search_terms=array();
	public $error_code=0;
	public $search_fields=null;
	public $search_type=null;
	public $exact_words=null;
	public $display_page=null;
	public $orderby=null;
	public $orderdir=null;
	public $results_count=null;
	protected $search_highlight_start="<span class='hl'>";
	protected $search_highlight_end="</span>";

	// count results
	// 
	// returns the number of results from the given query
	//
	// parameters
	// query - the query to evaluate
	// returns
	// count - the number of results generated
	
	public function count_results($query=null){
	
		//return 0;
		
		if($this->error_code==1){return null;}
		
		if($query==null){$query=$this->query;}	
		

		// check for a "count(*)"
		// if absent, add it in
		
		if(strpos($query,"count(*)")===false){
		
			$frpos=strpos($query,"from");
			
			$qu="select count(*) ".substr($query,$frpos);
			
			$query=$qu;
		
		}
		
		// remove LIMIT clause
		
		$limpos=strpos($query,"LIMIT");
		
		if($limpos!==0){
		
			$query=substr($query,0,$limpos-1);
		
		}
		
		// execute query
		
		$res=mysql_query($query);
		if(mysql_error()){echo(mysql_error());}
		
		$ar=mysql_fetch_assoc($res);
		
		$count=$ar["count(*)"];
		
		return $count;
	
	} // end count results
	
	// execute_query
	//
	// executes the given sql query
	// gives results as an associative array.
	//
	// parameters
	// query - an sql query
	// return
	// array - the results array
	
	public function execute_query($query=null){
	
		if($this->error_code==1){return null;}
		
		if($query==null){$query=$this->query;}
		
		//echo($query);
		
		$res=mysql_query($query);
		if(mysql_error()){echo(mysql_error());}
		
		$array=array();
		
		while($a=mysql_fetch_assoc($res)){$array[]=$a;}
		
		$this->results=$array;
		
		return $array;
	
	} // end execute_query
	

	// where_clause
	//
	// creates the "where" clause for the sql search
	//
	// parameters
	// search_query - the search query
	// search_columns - array of columns in which to search 
	// returns
	// where - the where clause
	
	public function where_clause($search_query,$search_columns){
	
		//$terms=$this->split_search($bool_terms);
		
		$where=" where ".$this->boolean_search($search_query, $search_columns);
		
		$this->where=$where;
		
		return $where;
	
	} // end where_clause

	// generate_query
	//
	// produces a full sql query from the given parameters.
	// You can have an unlimited number of linked tables
	//
	// parameters
	// where - where clause (previously generated)
	// table - table to search
	// return_columns - columns to return (array), or ALL for all
	// limit - limit statement
	// orderby - order by statement
	// join1 - join statement (left join, right join, join)
	// linked_table1 - linked table
	// on1 - on statement for linked_table1
	// return
	// query - the query
	
	public function generate_query($where,$table,$return_columns="ALL",$limit="",$orderby="",$count_only=false){
	
		$args=func_get_args();
		$stnd_args=6;
		$linked_args=3;
		
		$qu="select ";
		
		$qu_col=array();
		
		if((count($return_columns)==0)||(!is_array($return_columns))||($return_columns=="*")){$qu.="* ";}
		
		elseif($count_only){
		
			$qu.="count(*) ";
		
		}// end elseif
		
		else{
			
			// parse through the columns array
			
			foreach($return_columns as $col){
		
				$qu_col[]="`$col`";
		
			}// end foreach
		
			$qu.=implode(",",$qu_col);
			
		}// end else
		
		$qu.=" from `$table`";
		
		$num_args=count($args)-$stnd_args;
		
		if(($num_args>0)&&($num_args%$linked_args==0)){
		
			// handling of linked tables
			
			$num_linked=$num_args/$linked_args;
			
			for($i=$stnd_args;$i<($num_args+$stnd_args);$i+=$linked_args){
			
				$join=$args[$i];
				$table=$args[$i+1];
				$on=$args[$i+2];
				
				$qu.=" $join `$table` ON $on ";
			
			} // end for
		
		} // end if
		
		// add where clause
		
		$qu.=" ".$where;
		
		// add order and limit clauses
		if($orderby!=""){$qu.=" ORDER BY $orderby";}
		if($limit!=""){$qu.=" LIMIT ".$limit;}
		
		$this->query=$qu;
		
		return $query;
	
	}// end generate query


	
	// generate_order
	//
	// generates the order by clause
	//
	// parameters
	// fields - the table field(s) by which to sort (array)
	// directions - the sort direction(s) for each field
	// return
	// orderby - the order by clause
	
	public function generate_order($fields,$directions){
	
		if(count($fields)!==count($directions)){trigger_error("Linked Array Imbalance",E_USER_WARNING);}
		
		$stop=count($fields);
		$orderby=array();
		
		for($i=0;$i<$stop;$i++){
					
			$dir=$directions[$i];
			$fe=$fields[$i];
			
			if(($dir!="desc")&&($dir!="asc")){$dir="asc";}
			
			$orderby[]="`$fe` $dir";
		
		}// end for
				
		$orderby=implode(",",$orderby);
		
		return $orderby;
	
	} // end generate_order

	
	
	// search
	//
	// performs a basic search
	// 
	// parameters
	// terms - search terms
	// search_type - all_words, any_word, exact_phrase, boolean
	// search_columns - columns in which to search
	
	public function search($terms,$search_type,$search_columns){
	
		$boolterms=$this->booleanize($terms,$search_type);
		$where_clause=$this->where_clause($boolterms,$search_columns);
		$query=$this->generate_query($where,$return_columns,$table);
		$res=$this->execute_query($query);
		$highlight_results=$this->highlight($res,$highlight_columns);

	} // end search
	
	// booleanize
	//
	// Converts a search query into a boolean expression
	//
	// paramaters
	// search_query - the search terms (string)
	// search_type - all_words, any_words, exact_phrase, boolean, or regexp
	// returns
	// bool - the booleanized search query
	
	public function booleanize($search_query, $search_type){
		
		$search_query=strip_extra_spaces($search_query);
		$bool=$search_query;
		
		if($search_type=="any_word"){
			
			$bool=str_replace(" "," | ",$bool);
		
		}
		elseif($search_type=="exact_phrase"){
		
			$bool='"'.$bool.'"';
		
		}
		elseif($search_type=="regexp"){
		
			$bool='{'.$bool.'}';		
		
		}
		
		// standardize the boolean operators used in the search query
		
		$or_list=array("||"," OR ","+");
		$and_list=array("&&"," AND ","*");
		$not_list=array("-"," NOT ");
		$xor_list=array(" XOR ","^^");
		
		$bool=str_replace($or_list, " | ",$bool);
		$bool=str_replace($and_list," & ",$bool);
		$bool=str_replace($not_list," ! ",$bool);
		$bool=str_replace($xor_list," ^ ",$bool);
		
		$bool=strip_extra_spaces($bool);
		
		return $bool;
	
	}// end booleanize

	
	// bool_type
	//
	// determines if character is an operator,
	// parenthenses, or a letter
	// Used for "implied and" in boolean searches
	// 
	// parameters
	// char - the character to check
	// return
	// type - the type of character, as it relates to boolean searches
	
	protected function bool_type($char){

		if (($char=="(")){return "lp";}
		if (($char==")")){return "rp";}
		if (($char=="&") or ($char=="|") or ($char=="^")){return "o";}
		if ($char=="-"){return "n";}
		if ($char==" "){return "s";}
		return "l"; // letters, ect
		
	}// end bool_type
	

	// split_search
	//
	// splits a search into it's individual terms
	//
	// parameters
	// search_query - search query
	// underscore_phrases - causes spaces in phrases to be converted to underscores
	// @return $search_array array of search terms

	public function split_search($search_query="", $underscore_phrases=false){

		if ($search_query==""){$search_query==$this->search_query;}

		$search_query=strip_extra_spaces($search_query);
		$search_query=trim($search_query);

		$first_quo=array();
		$last_quo=array();
		$exact=array();

		$fi=-1;
		$la=-1;
		$i=true;

		// find the positions of the quotations
		while($i==true){

			$fi=strpos($search_query, "\"", $la+1);
		
			if ($fi!==false){
		
				$la=strpos($search_query, "\"", $fi+1);
			
				if ($la===false){break;}
			
				//$first_quo[]=$fi;
				//$last_quo[]=$la;

				$exact[]=trim(substr($search_query,$fi, $la-$fi+1));//($sea,$fi, $la-$fi+2);

			}// end if

			else{$i=false;}

		}// end while
		$exact_sv=array();

		foreach ($exact as $mm){
	
			$mm=trim($mm);
			$mm=str_replace(array(")","("),"",$mm);
			$exact_sv[]=$this->bool_transform($mm);
		
		}

		$search_query=str_replace($exact, $exact_sv, $search_query);
		//echo("|$sea|<br>");
	
		$find=array("(",")", "||","&&","!!","&","*","+","|","-","NOT", "AND","OR","!=","XOR","^","^^");
		$search_query=str_replace($find, " ",$search_query);
		$search_query=strip_extra_spaces($search_query);

		$array=explode(" ",trim($search_query));
		//print_out($array);

		if (!$underscore_phrases){
			$array=$this->inv_bool_transform($array);
		}// end if

		// remove repeated words
		$i=0;
		while($i<count($array)){

			$value=$array[$i];
			$j=$i+1;
			while($j<count($array)){

				if ($array[$j]==$value){unset($array[$j]);}
				$j++;
			
			}// end while

			$i++;
		}// end while (repeated words)

		/*
		echo("<pre>");
		print_r($search_query);
		print_r($array);
		echo("</pre>");
		*/

		return $array; 
		

	
	}// end split_search
	
	protected function bool_transform($query,$inv=false){

		$tx=array(")","(", " ",  "AND", "OR", "NOT","XOR","{","}","[","]","&","!","-","|","*","+","^","PROX","NEAR","'","_");
		$rx=array("<rp>","<lp>","<sp>","<and>","<or>","<not>","<xor>","<lc>", "<rc>","<lb>","<rb>","<amp>","<ex>","<neg>","<ver>", "<sta>","<pl>","<hat>","<prx>","<ner>","<sq>","<usc>");

		if(!$inv){$que=str_replace($tx,$rx,$query);}

		else{$que=str_replace($rx,$tx,$query);}

		return $que;

	}// end bool_transform

	protected function inv_bool_transform($query){return $this->bool_transform($query,true);}
	

	// generate_limit_from_page
	//
	// generates a limit statement for a paginated search
	//
	// parameters
	// page_num - the page number (integer)
	// items_per_page - the number of search results to be displayed on a page.
	// returns
	// limit - the limit statement
	
	public function generate_limit_from_page($page_num,$items_per_page){
		
		// page 1 - lim 30: 0, 30
		// page 2 - lim 30: 30,30
		// page 3 - lim 30: 60,30
		
		$start=($page_num-1)*$items_per_page;
		
		$limit="$start, $items_per_page";
		
		return $limit;
	
	} // end generate_limit_from_page
	


	
	// highlight
	//
	// highlights the search term(s) in the result set
	// 
	// parameters
	// results_array - result array
	// highlight_columns - array of columns to highlight
	// return
	// results_array - results, with keywords highlighted
	
	public function highlight($results_array,$highlight_columns="ALL"){
		
		global $table_content;
		
		$stop=count($results_array);
		
		// parse through the results array
		
		for($i=0;$i<$stop;$i++){
		
			$res=$results_array[$i];
			
			foreach($highlight_columns as $col){
			
				// for the body content, highlight it and truncate it
				
				if($col==$table_content){
				
					$results_array[$i][$col]=$this->highlight_text($results_array[$i][$col]);
				
				}// end if
				
				// for other fields, just highlight
				
				else{
				
					$results_array[$i][$col]=$this->highlight_single($results_array[$i][$col]);
				
				}// end else
			
			}// end foreach
			
		}// end for
				
		return $results_array;
	
	} // end highlight
	
	
	
	// highlight_single
	//
	// highlights the given text for the specified search terms
	//
	// parameters
	// text - the text to highlight.
	// return
	// text - the highlighted text
	
	protected function highlight_single($text){
	
		$terms=$this->search_terms;
		
		$sth=$this->search_highlight_start;
		$enh=$this->search_highlight_end;
		
		$sth_len=strlen($sth);
		$enh_len=strlen($enh);
		$text_len=strlen($text);
	
		//echo("$text<br>");
	
		// highlight each term
		
		foreach($terms as $term){
			
			$stop=false;
			$term_len=strlen($term);
			$ip=0;
			
			$cnt=0;
			
			$term=trim($term);
			$term=trim($term,"%_");
			

			
			while(!$stop){
							
				// search for the term in "text"
				
				$pos=stripos($text,$term,$ip);
			
				if($pos===false){
			
					$stop=true;
					

			
				}// end if
				else{
			
					$end_pos=$pos+$sth_len+$term_len;
					
					// insert the start and end tags
					$text=substr_replace($text,$sth,$pos,0);
					$text=substr_replace($text,$enh,$end_pos,0);
					
					$cnt++;
			
				}// end else
			
				$ip=$end_pos+$enh_len+1;
				if($ip>$text_len){$stop=true;}
				
				//echo("No more:|$term| ip:|$ip|count:|$cnt|<br>");
			
			} // end else
		
		}// end foreach
		
		return $text;
	
	}// end highlight_single
	
	// highlight_text
	//
	// highlights the text, then truncates it to show only the highlighted
	// words, in context with the rest of the text. Used for the body text
	// of the content.
	//
	// parameters
	// text - the text to highlight
	// return
	// text - the highlighted text
	
	public function highlight_text($text){
		

		$sth=$this->search_highlight_start;
		$enh=$this->search_highlight_end;
		
		$sth_len=strlen($sth);
		$enh_len=strlen($enh);
		
		$text=strip_tags($text);
		
		$text=$this->highlight_single($text);
		
		//echo("$text<br>");
		
		//echo("<xmp>$text</xmp>");
		
		$text_len=strlen($text);
	
		//echo("text_len:|$text_len|<br>");
		
		$new_text=array();
		$len=0;
		$ip=0;
		$ee=0;
		$stop=false;
		$nt_old="";
		
		//$stop=true;
		
		while(!$stop){
		
			// search for a start tag
	
			
			$start=strpos($text,$sth,$ip);
			if($start===false){
				$stop=true;
			}
			else{
			
				$stop2=false;
				
				//echo("start:|$start|<br>");
				
				// find the corresponding end tag
				
				// end tag position
				$end=strpos($text,$enh,$start);
				
				// position of space at least (x) characters ahead of $start
				
				$stst=$start-15;
				
				if($stst>0){
				
					$sst=substr($text,0,$stst);
					$ss=strrpos($sst," ");				
					
				}
				else{
				
					$ss=0;
				
				}
				

				
				while(!$stop2){
								
					// check for other highlighting tags within the same piece of text
					// expand the text segment, if nessessary

					
					// position of space at least (x) characters from $end
					error_reporting(E_ERROR);
					$ee=strpos($text," ",$end+25);	
					error_reporting(E_ALL - E_NOTICE);
					if($ee===false){$ee=$text_len;}
					
					// the new string segment
					$ntl=$ee-$ss+1;
					$nt=substr($text,$ss,$ntl);
					
					//echo("<br>bob - ss:|$ss|ee:|$ee|ntl:|$ntl| nt|$nt|<br><br>");
	
					
					if($end<$text_len){
					
						//error_reporting(E_ERROR);
						$start2=strpos($nt,$sth,$end-$ss);
						error_reporting(E_ALL - E_NOTICE);
					
					}
					else{
					
						$stop2=true;
						echo("here 1");
					
					}
					
					if($start2!==false){
					
						$end=strpos($text,$enh,($start2+$ss));
						echo("start2 here = |$start2| ");
					
					}// end if
					else{
					
						$stop2=true;
						echo("here 2");
					
					}// end else
					
					if(($end>=$text_len)||($ee==$text_len)){
					
						$stop2=true;
					
					}// end else

				}// eld while
			
			}// end else
			
			$len+=strlen($nt);
			
			if($len>300){$stop=true;}
			else{
				
				// save text segment to array
				// if it isn't a repeat
				
				if($nt!=$nt_old){
				
					$new_text[]=$nt;
					$nt_old=$nt;
				
				} // end if
				else{
				
					$len-=strlen($nt);
				
				} // end else
				
				$ip=$ee;
			
			}// end else
			
			$ip=$ee+1;
		
		}// end outer while
		
		$st=count($new_text);
		
		for($i=0;$i<$st;$i++){
		
			$tt=$new_text[$i];
			$len=strlen($tt);
			
			$st=strpos($tt,">");
			if(($st===false)||(($st+1)>=$sth_len)){$st=0;}
			$en=stripos($tt,"<");
			if(($en===false)||(($len-$en+1)>=$sth_len)){$en=$len-1;}
			
			$new_text[$i]=substr($tt,$st,$en-$st);
			
			echo("tt:|$tt| new: |{$new_text[$i]}|<br>");
		
		}
		
		// construct the truncated text
		$new_text="... ".implode(" ... ",$new_text). "...";
		if($new_text=="... ..."){$new_text="";}
		$new_text=strip_extra_spaces($new_text);
		
		return $new_text;
	
	} // end highlight text


	
	// explodeSearch
	//
	// separates the Search into separate search terms
	// also removes all logical operators and parenthenses from boolean search
	// used to generate search array for highlighting search terms
	//
	// parameters
	// search - search query
	// type - search type (all, any, phrase, boolean)
	// returns 
	// search_array - array of all search terms

	public function explode_search($search_query="",$type=""){
	
		$exploded=array();
		
		if($search_query==""){$search_query=$this->search_query;}
		if($type==""){$type=$this->search_type;}
	
		// all words or any words
		if (($type=="any_word")||($type=="all_words")){$exploded=explode(" ", $search_query);}
		
		// exact phrase and regular expression
		elseif (($type=="exact_phrase")||($type="regexp")){
		
			$exploded=array($search_query);			
		
		}// end if 
		
		// boolean expressions
		
		elseif($type=="boolean"){
		
			$exploded=$this->split_search($search);
			$exploded=str_replace("\\\"","",$exploded);
			$exploded=str_replace("%","",$exploded);			
		
		}// end elseif
		
		$this->search_terms=$exploded;
		
		return $exploded;

	}// end explodeSearch
	
	
	
	// bool_examine
	//
	// Examines query for places where an AND operator is implied
	// but is not included in the query.  Inserts an AND in the places
	// where it is implied.
	//
	// paramaters
	// search query - the search query
	// return 
	//
	
	protected function bool_examine($search_query){

		$i=1;

		$search=strip_extra_spaces($search_query);
		$len=strlen($search);
		
		while($i<$len){

			// locate the next space
			
			$pos=strpos($search, " ",$i);

			// IMPLIED AND

			if ($pos>=($len-1)){break;}

			// find the character types before and after this space

			$lo=substr($search,$pos-1,1);
			$hi=substr($search,$pos+1,1);
			$lo_type=$this->bool_type($lo);
			$hi_type=$this->bool_type($hi);

			// detect an implied AND, and insert an AND
			
			if ((($lo_type=="l") and ($hi=="(")) or (($hi_type=="l") and ($lo==")")) or (($lo_type=="l") and ($hi_type=="l")) or (($lo_type=="l")and($hi_type=="n")) or (($lo==")")and($hi=="(")) or (($lo==")")and($hi_type=="n"))){

				$find="$lo $hi";
				$rep="$lo & $hi";

				$search=str_replace($find, $rep, $search);
				$pos+=1;
				
				if (($hi_type!="rp")&&($hi_type!="lp")&&($lo_type!="rp")&&($lo_type!="lp")){$pos+=2;}

			}// end if

			$i=$pos+1;
			
		}// end while

		return $search;
		
	}// end bool_examine
	
	
	
	// boolean search
	//
	// converts a boolean expression into a sql query
	//
	// parameters
	// search_query - the (boolean) search query
	// columns - database table fields in which to search

	public function boolean_search($search_query,$columns){
		
		$search=str_replace("\\","",$search_query);

		$terms_u=$this->split_search($search, true);
		$terms=$this->split_search($search);

		$search=str_replace($terms, $terms_u, $search);

		$rep=array();
		$i=0;
		$stop=count($terms);
		while ($i<$stop){

			$rep[]=" ".$this->bool_query($terms[$i])." ";
			$terms[$i]=" ".$terms[$i]." ";
			$terms_u[$i]=" ".$terms_u[$i]." ";
			//echo("|$terms[$i]|<br>");
			$i++;
	
		}// end while

		$search=strip_extra_spaces($search);

		// prep the search string before converting it to query
		$ad_ser=array("(",")","{","}","[","]");
		$ad_rep=array(" ( ", " ) "," ( ", " ) "," ( ", " ) ");
		$search=str_replace($ad_ser, $ad_rep, $search);
		$search=strip_extra_spaces($search);
		
		//echo($search);

		$search=" $search ";
		
		// operator aliases
		$and=array("AND", "*","&&","&");
		$oR=array("OR","+","||","|");
		$NOT=array("NOT", "-","!=","-");
		$XOR=array("XOR","^^","^");

		$search=str_replace($XOR," ^ ",$search);
		$search=str_replace($and, " & ",$search);
		$search=str_replace($oR, " | ",$search);
		$search=str_replace($NOT, " - ",$search);

		// if only NOT searches, return null

		// count number of NOTs and ORs
		$cneg=count(explode("-",$search))-1;
		$cor=count(explode("|",$search))-1;
		$ct=count($terms);
		$ss=str_replace("| ","+",$search);

		if ((($ct==$cor+1)&&($cneg!=0))||($cneg==$ct)||(strpos($search,"| -")!==false)){
		
			return null;
		
		} // end if


		$search=$this->bool_examine($search);

		$query=str_replace($terms_u, $rep, $search);
		$query=strip_extra_spaces($query);

		$ser=array("&","|","-","^");
		$rer=array("AND","OR"," NOT ","XOR");
		$query=str_replace($ser,$rer, $query);

		$query=str_replace("\\\"","",$query);
		$query="($query)";
		
		// build the query for each collumn
		
		$qu=array();
		
		foreach($columns as $col){
		
			$qu[]=str_replace("<==>",$col,$query);
		
		}
		
		$query="(".implode(" OR ",$qu).")";
		
		return $query;
	
	}// end boolean_search



	// bool_query
	//
	// creates advanced search query for single term
	//
	// parameters
	// term - term to create query for
	// return
	// query - query for the term

	protected function bool_query($term){

		if(($this->search_type=="regexp")||(($this->search_type=="boolean")&&($term{0}=="{"))){
		
			$op="REGEXP";
			$wc="";
			$term=str_replace("\"","",$term);
			$term=trim($term,"{}");
		
		}
		else{
		
			$op="LIKE";
			$wc="%";
		
		}
		
		$term=trim($term);
		$term=mysql_real_escape_string($term);

		$wholeword=$this->exact_words;

		if (($wholeword!="")&&($op!="REGEXP")){

			return $this->whole_word_query($term);
		
		}// end if
		
		$query="(`<==>` $op '$wc$term$wc')";

		return $query;

	}// end bool_query



	// whole_word_query
	// builds the mySQL query for "whole word only" searches
	// @ param word - the word to be searched for
	// @ return qu - the (partial) query for a whole word

	protected function whole_word_query($word){

		if(($this->search_type=="regexp")||(($this->search_type=="boolean")&&($word{0}=="{"))){
		
			$op="REGEXP";
		
		} // end if
		else{
		
			$op="LIKE";
		
		} // end else
		
		$s=array("$word", "$word.", "$word,", "$word;" , "$word!", "$word:", "$word?", "$word\\'", "$word\"", "\"$word ","\\'$word ","\\'$word\\'","“$word ","$word”");

		$i=1;
		$qu="((`<==>` $op '% $word %')";
		while(!empty($s[$i])){

			$qu.=" OR (`<==>` $op '% ".$s[$i]."%')";
			$qu.=" OR (`<==>` $op '".$s[$i]."%')";
			$i+=1;
			
		} // end while

		$qu.="OR (`<==>` $op '".""."$word %'))";

		return $qu;
		
	}// end whole_word_query

}// end class

?>
